@@ -24,6 +24,9 @@ class hypothesis:
 class instapaper:
     export_path: Paths = ''
 
+class smscalls:
+    export_path: Paths = ''
+
 class pocket:
     export_path: Paths = ''
 
@@ -31,7 +34,12 @@ class github:
     export_path: Paths = ''
 
 class reddit:
-    export_path: Paths = ''
+    class rexport:
+        export_path: Paths = ''
+    class pushshift:
+        export_path: Paths = ''
+    class gdpr:
+        export_path: Paths = ''
 
 class endomondo:
     export_path: Paths = ''
@@ -45,6 +53,9 @@ class bluemaestro:
 class stackexchange:
     export_path: Paths = ''
 
+class goodreads:
+    export_path: Paths = ''
+
 class pinboard:
     export_dir: Paths = ''
 
@@ -69,3 +80,18 @@ class time:
 
 class orgmode:
     paths: Paths
+
+
+class arbtt:
+    logfiles: Paths
+
+
+from typing import Optional
+class commits:
+    emails: Optional[Sequence[str]]
+    names: Optional[Sequence[str]]
+    roots: Sequence[PathIsh]
+
+
+class pdfs:
+    paths: Paths
