@@ -18,6 +18,7 @@ def import_file(p: PathIsh, name: Optional[str]=None) -> types.ModuleType:
         name = p.stem
     import importlib.util
     spec = importlib.util.spec_from_file_location(name, p)
+    assert spec is not None, f"Fatal error; Could not create module spec from {name} {p}"
     foo = importlib.util.module_from_spec(spec)
     loader = spec.loader; assert loader is not None
     loader.exec_module(foo) # type: ignore[attr-defined]
@@ -70,18 +71,62 @@ def group_by_key(l: Iterable[T], key: Callable[[T], K]) -> Dict[K, List[T]]:
 def _identity(v: T) -> V:
     return cast(V, v)
 
-def make_dict(l: Iterable[T], key: Callable[[T], K], value: Callable[[T], V]=_identity) -> Dict[K, V]:
-    res: Dict[K, V] = {}
-    for i in l:
+
+# ugh. nothing in more_itertools?
+def ensure_unique(
+        it: Iterable[T],
+        *,
+        key: Callable[[T], K],
+        value: Callable[[T], V]=_identity,
+        key2value: Optional[Dict[K, V]]=None
+) -> Iterable[T]:
+    if key2value is None:
+        key2value = {}
+    for i in it:
         k = key(i)
         v = value(i)
-        pv = res.get(k, None) # type: ignore
+        pv = key2value.get(k, None)  # type: ignore
         if pv is not None:
             raise RuntimeError(f"Duplicate key: {k}. Previous value: {pv}, new value: {v}")
-        res[k] = v
+        key2value[k] = v
+        yield i
+
+
+def test_ensure_unique() -> None:
+    import pytest  # type: ignore
+    assert list(ensure_unique([1, 2, 3], key=lambda i: i)) == [1, 2, 3]
+
+    dups = [1, 2, 1, 4]
+    # this works because it's lazy
+    it = ensure_unique(dups, key=lambda i: i)
+
+    # but forcing throws
+    with pytest.raises(RuntimeError, match='Duplicate key'):
+        list(it)
+
+    # hacky way to force distinct objects?
+    list(ensure_unique(dups, key=lambda i: object()))
+
+
+def make_dict(
+        it: Iterable[T],
+        *,
+        key: Callable[[T], K],
+        value: Callable[[T], V]=_identity
+) -> Dict[K, V]:
+    res: Dict[K, V] = {}
+    uniques = ensure_unique(it, key=key, value=value, key2value=res)
+    for _ in uniques:
+        pass  # force the iterator
     return res
 
 
+def test_make_dict() -> None:
+    it = range(5)
+    d = make_dict(it, key=lambda i: i, value=lambda i: i % 2)
+    assert d == {0: 0, 1: 1, 2: 0, 3: 1, 4: 0}
+
+
 Cl = TypeVar('Cl')
 R = TypeVar('R')
 
@@ -164,7 +209,9 @@ def get_files(
                 warnings.warn(f"{caller()}: treating {gs} as glob path. Explicit glob={glob} argument is ignored!")
             paths.extend(map(Path, do_glob(gs)))
         elif src.is_dir():
-            gp: Iterable[Path] = src.glob(glob) # todo not sure if should be recursive?
+            # todo not sure if should be recursive?
+            # note: glob='**/*.ext' works without any changes.. so perhaps it's ok as it is
+            gp: Iterable[Path] = src.glob(glob)
             paths.extend(gp)
         else:
             if not src.is_file():
@@ -214,20 +261,44 @@ if TYPE_CHECKING:
 
     mcachew: McachewType
 
-# TODO set default cache dir here instead?
-# todo ugh. I think it needs doublewrap, otherwise @mcachew without args doesn't work
-def mcachew(*args, **kwargs): # type: ignore[no-redef]
+
+_CACHE_DIR_NONE_HACK = Path('/tmp/hpi/cachew_none_hack')
+"""See core.cachew.cache_dir for the explanation"""
+
+
+_cache_path_dflt = cast(str, object())
+# TODO I don't really like 'mcachew', just 'cache' would be better... maybe?
+# todo ugh. I think it needs @doublewrap, otherwise @mcachew without args doesn't work
+# but it's a bit problematic.. doublewrap works by defecting if the first arg is callable
+# but here cache_path can also be a callable (for lazy/dynamic path)... so unclear how to detect this
+def mcachew(cache_path=_cache_path_dflt, **kwargs): # type: ignore[no-redef]
     """
     Stands for 'Maybe cachew'.
     Defensive wrapper around @cachew to make it an optional dependency.
     """
+    if cache_path is _cache_path_dflt:
+        # wasn't specified... so we need to use cache_dir
+        from .cachew import cache_dir
+        cache_path = cache_dir()
+
+    if isinstance(cache_path, (str, Path)):
+        try:
+            # check that it starts with 'hack' path
+            Path(cache_path).relative_to(_CACHE_DIR_NONE_HACK)
+        except:
+            pass # no action needed, doesn't start with 'hack' string
+        else:
+            # todo show warning? tbh unclear how to detect when user stopped using 'old' way and using suffix instead?
+            # if it does, means that user wanted to disable cache
+            cache_path = None
     try:
         import cachew
     except ModuleNotFoundError:
         warnings.warn('cachew library not found. You might want to install it to speed things up. See https://github.com/karlicoss/cachew')
         return lambda orig_func: orig_func
     else:
-        return cachew.cachew(*args, **kwargs)
+        kwargs['cache_path'] = cache_path
+        return cachew.cachew(**kwargs)
 
 
 @functools.lru_cache(1)
@@ -282,21 +353,12 @@ class classproperty(Generic[_R]):
 tzdatetime = datetime
 
 
-fromisoformat: Callable[[str], datetime]
-import sys
-if sys.version_info[:2] >= (3, 7):
-    # prevent mypy on py3.6 from complaining...
-    fromisoformat_real = datetime.fromisoformat
-    fromisoformat = fromisoformat_real
-else:
-    from .py37 import fromisoformat
-
-
 # TODO doctests?
 def isoparse(s: str) -> tzdatetime:
     """
     Parses timestamps formatted like 2020-05-01T10:32:02.925961Z
     """
+    from .compat import fromisoformat
     # TODO could use dateutil? but it's quite slow as far as I remember..
     # TODO support non-utc.. somehow?
     assert s.endswith('Z'), s
@@ -507,7 +569,13 @@ def test_guess_datetime() -> None:
     # TODO test @property?
 
 
-def asdict(thing) -> Json:
+def is_namedtuple(thing: Any) -> bool:
+    # basic check to see if this is namedtuple-like
+    _asdict = getattr(thing, '_asdict', None)
+    return _asdict and callable(_asdict)
+
+
+def asdict(thing: Any) -> Json:
     # todo primitive?
     # todo exception?
     if isinstance(thing, dict):
@@ -515,16 +583,45 @@ def asdict(thing) -> Json:
     import dataclasses as D
     if D.is_dataclass(thing):
         return D.asdict(thing)
-    # must be a NT otherwise?
-    # todo add a proper check.. ()
-    return thing._asdict()
+    if is_namedtuple(thing):
+        return thing._asdict()
+    raise TypeError(f'Could not convert object {thing} to dict')
 
 
-# todo not sure about naming
-def to_jsons(it) -> Iterable[Json]:
-    from .error import error_to_json # prevent circular import
-    for r in it:
-        if isinstance(r, Exception):
-            yield error_to_json(r)
+
+
+datetime_naive = datetime
+datetime_aware = datetime
+
+
+def assert_subpackage(name: str) -> None:
+    # can lead to some unexpected issues if you 'import cachew' which being in my/core directory.. so let's protect against it
+    # NOTE: if we use overlay, name can be smth like my.origg.my.core.cachew ...
+    assert name == '__main__' or 'my.core' in name, f'Expected module __name__ ({name}) to be __main__ or start with my.core'
+
+
+# https://stackoverflow.com/a/10436851/706389
+from concurrent.futures import Future, Executor
+class DummyExecutor(Executor):
+    def __init__(self, max_workers: Optional[int]=1) -> None:
+        self._shutdown = False
+        self._max_workers = max_workers
+
+    def submit(self, fn, *args, **kwargs) -> Future:
+        if self._shutdown:
+            raise RuntimeError('cannot schedule new futures after shutdown')
+
+        f: Future[Any] = Future()
+        try:
+            result = fn(*args, **kwargs)
+        except KeyboardInterrupt:
+            raise
+        except BaseException as e:
+            f.set_exception(e)
         else:
-            yield asdict(r)
+            f.set_result(result)
+
+        return f
+
+    def shutdown(self, wait: bool=True) -> None:
+        self._shutdown = True
