@@ -10,9 +10,8 @@ REQUIRES = [
 from collections import Counter
 from datetime import date, datetime
 from functools import lru_cache
-from itertools import groupby, islice
-from pathlib import Path
-from typing import Dict, Iterator, List, NamedTuple, Optional, Tuple
+from itertools import groupby
+from typing import Iterator, NamedTuple, Optional
 
 from more_itertools import seekable
 import pytz
@@ -141,7 +140,7 @@ def localize(dt: datetime) -> tzdatetime:
 
 from ...core import stat, Stats
 def stats() -> Stats:
-    from ...core.common import fromisoformat
+    from ...core.compat import fromisoformat
     # TODO not sure what would be a good stat() for this module...
     # might be nice to print some actual timezones?
     # there aren't really any great iterables to expose
